package org.openjdk.gcbench.retain;

import org.openjdk.jmh.annotations.*;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Optional;
import java.util.WeakHashMap;
import java.util.concurrent.TimeUnit;

@Warmup(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Measurement(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Fork(1)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.NANOSECONDS)
@Threads(Threads.MAX)
@State(Scope.Benchmark)
public class WeakHashMaps {

    Object retain;

    @Param({"100000"})
    private int size;

    @Setup
    public void setup() {
        WeakHashMap<Object, Object> map = new WeakHashMap<>();
        for (int c = 0; c < size; c++) {
            Object v = new Object();
            // deadly-embrace to keep weak key strongly reachable;
            // also make sure the value is one dereference forward, in order
            // to let VM discover the reference first, not the referent.
            map.put(v, Optional.of(v));
        }
        retain = map;
    }

    @Benchmark
    public Object test() {
        // allocation pressure to trigger GCs
        return new Object();
    }

}
